ROCS_DEFAULT_VALUES = dict(
    MAX_STEREO=0,
    MAX_CPUS=4,
    SAVE_ROCS_OVERLAYS=False,
    ENUM_STEREO=False,
    MAX_CONFS=200,
    EWINDOW=10,
    ROCS_OVERLAYS_DIR=None,
    ROCS_OVERLAYS_PREFIX="",
    PROTEIN_NEG_VOL_FILE=None,
    LIGAND_NEG_VOL_FILE=None,
    ROCS_INPUT=None,
    INPUT_TYPE=None,
    CUSTOM_CFF="",
    SHAPE_WEIGHT=None,
    COLOR_WEIGHT=None,
    NEGATIVE_VOLUME=False,
)